/*
 * Copyright 2015 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jetbrains.android.anko.utils

import java.io.File
import java.io.FileFilter

public class JarFileFilter : FileFilter {
    override fun accept(path: File): Boolean {
        return path.isFile() && path.getName().toLowerCase().endsWith(".jar")
    }
}

public class AndroidVersionDirectoryFilter : FileFilter {
    override fun accept(path: File): Boolean {
        return path.isDirectory() && !path.isHidden() &&
                path.getName().matches("^[0-9]+s?$".toRegex()) &&
                path.listFiles { it.name.endsWith(".jar") }?.isNotEmpty() ?: false
    }
}
